/* ffaToFa - convert Greg Schulers .ffa fasta files to our .fa files */
#include "common.h"
#include "portable.h"
#include "linefile.h"
#include "hash.h"
#include "hCommon.h"

void usage()
/* Explain usage and exit. */
{
errAbort(
   "ffaToFa convert Greg Schuler .ffa fasta files to UCSC .fa fasta files\n"
   "usage:\n"
   "    ffaToFa ffaDir faDir out.tab\n"
   "where ffaDir is directory full of .ffa files, faDir is where you want\n"
   "to put the corresponding .fa files, and out.tab is a table that\n"
   "translates from one name to the other.\n");
}

void gsToUcsc(char *gsName, char *ucscName)
/* Convert from 
 *    AC020585.5~1 Fragment 1 of 29 (AC020585.5:1..1195)
 * to
 *    AC020585.5_1_1195
 */
{
char *s, *e, *d;
int size;

/* Copy in accession and version. */
d = ucscName;
s = strchr(gsName, '(');
if (s == NULL)
    errAbort("No '(' in .ffa style name: %s", gsName);
s += 1;
e = strchr(s, ':');
if (e == NULL)
    errAbort("No ':' in .ffa style name: %s", gsName);
size = e - s;
memcpy(d, s, size);
d += size;
*d++ = '_';

s = e+1;
e = strchr(s, '.');
if (e == NULL || e[1] != '.')
    {
    errAbort("Mangled .ffa style name: %s", gsName);
    }
size = e - s;
memcpy(d, s, size);
d += size;
*d++ = '_';

s = e+2;
e = strchr(s, ')');
if (e == NULL)
    errAbort("No ')' in .ffa style name: %s", gsName);
size = e - s;
memcpy(d, s, size);
d += size;
*d++ = 0;
return;
}

void ffaToFa(char *inDir, char *outDir, char *outTabName)
/* convert Greg Schulers .ffa fasta files to our .fa files */
{
char path[512], dir[256], file[128], ext[64];
struct slName *inList, *inEl;
char *inName;
int inCount;
struct lineFile *in;
FILE *out = NULL, *tab;
int lineSize;
char *line;
char ucscName[128];
static char lastPath[128];
char ucscFa[256];
int outFileCount = 0;
struct hash *uniq = newHash(16);

inList = listDir(inDir, "*.ffa");
inCount = slCount(inList);
if (inCount <= 0)
    errAbort("No .ffa files in %s", inDir);
tab = mustOpen(outTabName, "w");
printf("Converting %d files", inCount);
fflush(stdout);
for (inEl = inList; inEl != NULL; inEl = inEl->next)
    {
    inName = inEl->name;
    sprintf(path, "%s/%s", inDir, inName);
    in = lineFileOpen(path, TRUE);
    while (lineFileNext(in, &line, &lineSize))
	{
	if (line[0] == '>')
	    {
	    gsToUcsc(line+1, ucscName);
	    faRecNameToFaFileName(outDir, ucscName, path);
	    if (!sameString(path, lastPath))
		{
		strcpy(lastPath, path);
		carefulClose(&out);
		if (hashLookup(uniq, path))
		    {
		    warn("\nDuplicate %s in %s/%s ignoring all but first", 
		    	ucscName, inDir,inName);
		    }
		else
		    {
		    hashAdd(uniq, path, NULL);
		    out = mustOpen(path, "w");
		    ++outFileCount;
		    if ((outFileCount&3) == 0)
			{
			putc('.', stdout);
			fflush(stdout);
			}
		    }
		}
	    if (out != NULL)
		{
		fprintf(out, ">%s\n", ucscName);
		fprintf(tab, "%s\t%s\n", ucscName, line+1);
		}
	    }
	else
	    {
	    if (out != NULL)
		{
		fputs(line, out);
		fputc('\n', out);
		}
	    }
	}
    lineFileClose(&in);
    }
fclose(tab);
carefulClose(&out);
printf("Made %d .fa files in %s\n", outFileCount, outDir);
}

int main(int argc, char *argv[])
/* Process command line. */
{
if (argc != 4)
    usage();
ffaToFa(argv[1], argv[2], argv[3]);
return 0;
}
