/* pslSort - merge and sort psCluster .psl output files. */
#include "common.h"
#include "portable.h"
#include "linefile.h"
#include "memalloc.h"
#include "localmem.h"
#include "psl.h"


void usage()
/* Explain usage and exit. */
{
errAbort(
   "pslSort - merge and sort psCluster .psl output files\n"
   "usage:\n"
   "  pslSort outFile tempDir inDir(s)\n"
   "This will sort all of the .psl files in the directories\n"
   "inDirs in two stages - first into temporary files in tempDir\n"
   "and second into outFile.  The device on tempDir needs to have\n"
   "enough space (typically 15-20 gigabytes if processing whole genome)\n"
   );
}

void makeMidName(char *tempDir, int ix, char *retName)
/* Return name of temp file of given index. */
{
sprintf(retName, "%s/tmp%d.psl", tempDir, ix);
}

struct midFile
/* Info on an intermediate file - a sorted chunk. */
    {
    struct midFile *next;	/* Next in list. */
    struct lineFile *lf;        /* Associated file. */
    struct psl *psl;            /* Current record. */
    };

struct psl *nextPsl(struct lineFile *lf)
/* Read next line from file and convert it to psl.  Return
 * NULL at eof. */
{
char *line;
int lineSize;
char *words[32];
int wordCount;
struct psl *psl;

if (!lineFileNext(lf, &line, &lineSize))
    {
    //warn("File %s appears to be incomplete\n", lf->fileName);
    return NULL;
    }
wordCount = chopTabs(line, words);
if (wordCount == 21)
    {
    return pslLoad(words);
    }
else
    {
    warn("Bad line %d of %s", lf->lineIx, lf->fileName);
    return NULL;
    }
}


struct psl *nextLmPsl(struct lineFile *lf, struct lm *lm)
/* Read next line from file and convert it to psl.  Return
 * NULL at eof. */
{
char *line;
int lineSize;
char *words[32];
int wordCount;
struct psl *psl;

if (!lineFileNext(lf, &line, &lineSize))
    return NULL;
wordCount = chopTabs(line, words);
if (wordCount == 21)
    {
    return pslLoadLm(words, lm);
    }
else
    {
    warn("Bad line %d of %s", lf->lineIx, lf->fileName);
    return NULL;
    }
}




void pslSort2(char *outFile, char *tempDir)
/* Do second step of sort - merge all sorted files in tempDir
 * to final. */
{
char fileName[512];
struct slName *tmpList, *tmp;
struct midFile *midList = NULL, *mid;
int aliCount = 0;
FILE *f = mustOpen(outFile, "w");


pslWriteHead(f);
tmpList = listDir(tempDir, "tmp*.psl");
if (tmpList == NULL)
    errAbort("No tmp*.psl files in %s\n", tempDir);
for (tmp = tmpList; tmp != NULL; tmp = tmp->next)
    {
    sprintf(fileName, "%s/%s", tempDir, tmp->name);
    AllocVar(mid);
    mid->lf = pslFileOpen(fileName);
    slAddHead(&midList, mid);
    }
printf("writing %s", outFile);
fflush(stdout);
/* Write out the lowest sorting line from mid list until done. */
for (;;)
    {
    struct midFile *bestMid = NULL;
    if ( (++aliCount & 0xffff) == 0)
	{
	printf(".");
	fflush(stdout);
	}
    for (mid = midList; mid != NULL; mid = mid->next)
	{
	if (mid->lf != NULL && mid->psl == NULL)
	    {
	    if ((mid->psl = nextPsl(mid->lf)) == NULL)
		lineFileClose(&mid->lf);
	    }
	if (mid->psl != NULL)
	    {
	    if (bestMid == NULL || pslCmpQuery(&mid->psl, &bestMid->psl) < 0)
		bestMid = mid;
	    }
	}
    if (bestMid == NULL)
	break;
    pslTabOut(bestMid->psl, f);
    pslFree(&bestMid->psl);
    }
printf("\n");
fclose(f);

/* The followint really shouldn't be necessary.... */
for (mid = midList; mid != NULL; mid = mid->next)
    lineFileClose(&mid->lf);

printf("Cleaning up temp files\n");
for (tmp = tmpList; tmp != NULL; tmp = tmp->next)
    {
    sprintf(fileName, "%s/%s", tempDir, tmp->name);
    remove(tmp->name);
    }
}

void pslSort(char *outFile, char *tempDir, char *inDirs[], int inDirCount)
/* Do the two step sort. */
{
int i;
struct slName *fileList = NULL, *name;
char *inDir;
struct slName *dirDir, *dirFile;
char fileName[512];
int fileCount;
int totalFilesProcessed = 0;
int filesPerMidFile;
int midFileCount = 0;
FILE *f;
struct lineFile *lf;

/* Figure out how many files to process. */
for (i=0; i<inDirCount; ++i)
    {
    inDir = inDirs[i];
    dirDir = listDir(inDir, "*.psl");
    printf("%s with %d files\n", inDir, slCount(dirDir));
    for (dirFile = dirDir; dirFile != NULL; dirFile = dirFile->next)
	{
	sprintf(fileName, "%s/%s", inDir, dirFile->name);
	name = newSlName(fileName);
	slAddHead(&fileList, name);
	}
    slFreeList(&dirDir);
    }
printf("%d files in %d dirs\n", slCount(fileList), inDirCount);
slReverse(&fileList);
fileCount = slCount(fileList);
filesPerMidFile = round(sqrt(fileCount));
printf("Got %d files %d files per mid file\n", fileCount, filesPerMidFile);

/* Read in files a group at a time, sort, and write merged, sorted
 * output of one group. */
name = fileList;
while (totalFilesProcessed < fileCount)
    {
    int filesInMidFile = 0;
    struct psl *pslList = NULL, *psl;
    int lfileCount = 0;
    struct lm *lm = lmInit(256*1024);

    for (filesInMidFile = 0; filesInMidFile < filesPerMidFile && name != NULL;
    	++filesInMidFile, ++totalFilesProcessed, name = name->next)
	{
	boolean gotEnd = FALSE;
	printf("Reading %s (%d of %d)\n", name->name, totalFilesProcessed+1, fileCount);
	lf = pslFileOpen(name->name);
	while ((psl = nextLmPsl(lf, lm)) != NULL)
	    {
	    ++lfileCount;
	    slAddHead(&pslList, psl);
	    //pslFree(&psl);
	    }
	lineFileClose(&lf);
	}
    slSort(&pslList, pslCmpQuery);
    makeMidName(tempDir, midFileCount, fileName);
    printf("Writing %s\n", fileName);
    f = mustOpen(fileName, "w");
    pslWriteHead(f);
    for (psl = pslList; psl != NULL; psl = psl->next)
	{
	pslTabOut(psl, f);
	}
    fclose(f);
    pslList = NULL;
    lmCleanup(&lm);
    printf("lfileCount %d\n", lfileCount);
    ++midFileCount;
    }
pslSort2(outFile, tempDir);
}

int main(int argc, char *argv[])
/* Process command line. */
{
if (argc < 4)
    usage();
pslSort(argv[1], argv[2], &argv[3], argc-3);
}
   
