/* diGraph - Directed graph routines. */
#ifndef DIGRAPH_H
#define DIGRAPH_H

struct diGraph
/* A directed graph. */
    {
    struct dgNode *nodeList;
    struct hash *nodeHash;
    };

struct dgNode
/* A node on a directed graph */
    {
    struct dgNode *next;   /* Next in master node list. */
    char *name;            /* Name of node. */
    void *val;             /* Value of node. */
    struct dgNodeRef *nextList;  /* Ways out of node. */
    struct dgNodeRef *prevList;  /* Ways into node. */
    int topoOrder;      /* Topographical order after call to dgTopoSort. */
    int component;      /* Connected component number after call to dgConnectedComponents. */
    bool visited;       /* Visit order in depth first traversal. */
    bool conn;		  /* Flag for connection algorithms. */
    };

struct dgNodeRef
/* A reference to a node. */
    {
    struct dgNodeRef *next;	/* Next in list. */
    struct dgNode *node;        /* Pointer to node. */
    };

struct diGraph *dgNew();
/* Return a new directed graph object. */

struct dgNode *dgAddNode(struct diGraph *dg, char *name, void *val);
/* Create new node in graph. */

struct dgNode *dgAddNumberedNode(struct diGraph *dg, int id, void *val);
/* Create new node with a number instead of a name. */
 
struct dgNode *dgFindNode(struct diGraph *dg, char *name);
/* Find existing node in graph. */

struct dgNode *dgFindNumberedNode(struct diGraph *dg, int id);
/* Find node given number. */

void *dgNodeVal(struct dgNode *node);
/* Return value associated with node. */

void *dgNodeName(struct dgNode *node);
/* Return name associated with node. */

int dgNodeNumber(struct dgNode *node);
/* Return number of node.  (Will likely return 0 if node
 * was added with a name rather than a number). */

void dgConnect(struct diGraph *dg, struct dgNode *a, struct dgNode *b);
/* Connect node a to node b. */

void dgDisconnect(struct diGraph *dg, struct dgNode *a, struct dgNode *b);
/* Disconnect nodes a and b. */

boolean dgDirectlyFollows(struct diGraph *dg, struct dgNode *a, struct dgNode *b);
/* Return TRUE if b directly follows a. */

boolean dgPathExists(struct diGraph *dg, struct dgNode *a, struct dgNode *b);
/* Return TRUE if there's a path from a to b. */

struct dgNodeRef *dgFindConnectedToNode(struct diGraph *dg, struct dgNode *a);
/* Return reference list of all nodes connected to a, including a.
 * slFreeList this list when done. */

struct dgNodeRef *dgFindNewConnected(struct diGraph *dg, struct dgNode *a);
/* Find a connected component guaranteed not to be covered before 
 * that includes a. */

struct dgNodeRef *dgFindNextConnected(struct diGraph *dg);
/* Return list of nodes that make up next connected component,
 * or NULL if no more components.  slFreeList this when
 * done.  Call "dgClearConnFlags" before first call to this.
 * Do not call dgFindConnectedToNode between dgFindFirstConnected 
 * and this as they use the same flag variables to keep track of
 * what vertices are used. */

int dgConnectedComponents(struct diGraph *dg);
/* Count number of connected components and set component field
 * of each node to reflect which component it is in. */

void dgTopoSort(struct diGraph *dg);
/* Fill in topological order of nodes. */

boolean dgHasCycles(struct diGraph *dg);
/* Return TRUE if directed graph has cycles. */

void dgClearConnFlags(struct diGraph *dg);
/* Clear out conn flags. */

void dgClearVisitFlags(struct diGraph *dg);
/* Clear out visit order flags. */

struct dgNodeRef *dgFindNodeInRefList(struct dgNodeRef *refList, struct dgNode *node);
/* Return reference to node if in list, or NULL if not. */

void dgRemoveFromRefList(struct dgNodeRef **pRefList, struct dgNode *node);
/* Remove reference to node from list. */

struct dgNodeRef *dgNextList(struct dgNode *node);
/* Return list of nodes that follow node. Don't free (or rearrange)
 * this list please. */

struct dgNodeRef *dgPrevList(struct dgNode *node);
/* Return list of nodes that precede node. Don't free (or rearrange)
 * this list please. */

void dgDumpGraph(struct diGraph *dg, FILE *out, boolean hideIsolated);
/* Dump info on graph to output. */

#endif /* DIGRAPH_H */

